@@ -10,10 +10,10 @@ class StorageFactory:
     STORAGE_METHOD_MONGODB = "mongodb"
 
     @staticmethod
-    def getStorage(storage_config):
-        if storage_config.STORAGE_METHOD == StorageFactory.STORAGE_METHOD_MONGODB:
-            return MongoDbStorage(storage_config)
+    def getStorage(mcrit_config):
+        if mcrit_config.STORAGE_CONFIG.STORAGE_METHOD == StorageFactory.STORAGE_METHOD_MONGODB:
+            return MongoDbStorage(mcrit_config)
         # Alchemy not supported right now
         # if storage_config.STORAGE_METHOD == StorageFactory.STORAGE_METHOD_ALCHEMY:
         #     return AlchemyStorage(storage_config)
-        return MemoryStorage(storage_config)
+        return MemoryStorage(mcrit_config)
