@@ -5,6 +5,8 @@ from mcrit.minhash.MinHash import MinHash
 
 if TYPE_CHECKING: # pragma: no cover
     from mcrit.config.StorageConfig import StorageConfig
+    from mcrit.config.McritConfig import McritConfig
+    from mcrit.config.MinHashConfig import MinHashConfig
     from mcrit.storage.FunctionEntry import FunctionEntry
     from mcrit.storage.MatchingCache import MatchingCache
     from mcrit.storage.MemoryStorage import MemoryStorage
@@ -27,7 +29,7 @@ class StorageInterface:
     _config: "StorageConfig"
     _band_projection: None
 
-    def __init__(self, config: "StorageConfig") -> None:
+    def __init__(self, config: "McritConfig") -> None:
         """Set the StorageConfig, sets up an empty Storage or loads existing data, ensures indexing,
         and ensures the existence of an Family with name \"\" and family_id 0.
 
@@ -38,14 +40,17 @@ class StorageInterface:
             AssertionError: If family_id 0 already exists, but does not refer to Family \"\".
         """
         self._config = config
+        self._storage_config = config.STORAGE_CONFIG
+        self._minhash_config = config.MINHASH_CONFIG
         self._band_projection = None
 
     # -> Set[function_id]
-    def getCandidatesForMinHash(self, minhash: "MinHash") -> Set[int]:
+    def getCandidatesForMinHash(self, minhash: "MinHash", band_matches_required=1) -> Set[int]:
         """Given a MinHash, return all candidates from all matching bands.
 
         Args:
             minhash: a MinHash
+            band_matches_required: the number of bands a minhash needs to match before being considered a candidate
 
         Returns:
             candidates: a set of function_ids
@@ -53,11 +58,12 @@ class StorageInterface:
         raise NotImplementedError
 
     # -> Dict[function_id, Set[function_id]]
-    def getCandidatesForMinHashes(self, function_id_to_minhash: Dict[int, "MinHash"]) -> Dict[int, Set[int]]:
+    def getCandidatesForMinHashes(self, function_id_to_minhash: Dict[int, "MinHash"], band_matches_required=1) -> Dict[int, Set[int]]:
         """Given MinHashes by function_id, return all candidates from all matching bands.
 
         Args:
             function_id_to_minhash: a dict mapping a function_id to a MinHash.
+            band_matches_required: the number of bands a minhash needs to match before being considered a candidate
 
         Returns:
             candidates: a dict mapping a function_id to a set of candidate function_ids.
@@ -102,13 +108,14 @@ class StorageInterface:
         """
         raise NotImplementedError
 
-    def addSmdaReport(self, smda_report: "SmdaReport") -> Optional["SampleEntry"]:
+    def addSmdaReport(self, smda_report: "SmdaReport", isQuery=False) -> Optional["SampleEntry"]:
         """Add a SMDA report to storage to create its sample representation and return corresponding SampleEntry object.
         This also adds the sample's functions. If the sample's family is not in storage yet, it will be added.
         If a sample with the same SHA256 already exists in storage, smda_report is not added and None is returned.
 
         Args:
             smda_report: the SmdaReport to add to the storage
+            isQuery: True if this SmdaReport is associated with a matching query and its data shall not be persisted in the main DB
 
         Returns:
             A SampleEntry corresponding to smda_report or None, if SmdaReport was already added.
@@ -153,6 +160,30 @@ class StorageInterface:
         """
         raise NotImplementedError
 
+    def modifyFamily(self, family_id: int, update_information: dict) -> bool:
+        """Update a family from the storage
+
+        Args:
+            family_id: the id of the sample to modify
+            update_information: a dictionary with update information for fields (family_name, is_library)
+
+        Returns:
+            True if family_id was contained in the storage and updated successfully, False otherwise
+        """
+        raise NotImplementedError
+
+    def modifySample(self, sample_id: int, update_information: dict) -> bool:
+        """Update a sample from the storage
+
+        Args:
+            sample_id: the id of the sample to modify
+            update_information: a dictionary with update information for fields (family_name, version, component, is_library)
+
+        Returns:
+            True if sample_id was contained in the storage and updated successfully, False otherwise
+        """
+        raise NotImplementedError
+
     def deleteSample(self, sample_id: int) -> bool:
         """Remove a sample from the storage, also removes all functions of the sample.
         All minhashes will be removed from the bands.
@@ -244,6 +275,17 @@ class StorageInterface:
         """
         raise NotImplementedError
 
+    def getFunctionIdsBySampleId(self, sample_id: int) -> Optional[List["int"]]:
+        """For a given sample_id, get all corresponding function_ids.
+
+        Args:
+            sample_id: a sample_id
+
+        Returns:
+            A list of int or None, if sample_id does not exist
+        """
+        raise NotImplementedError
+
     def getFunctionById(self, function_id: int, with_xcfg=False) -> Optional["FunctionEntry"]:
         """Given a function_id, return the respective FunctionEntry or None, if function_id is not contained otherwise.
 
@@ -508,12 +550,13 @@ class StorageInterface:
     def getStats(self) -> Dict[str, Union[int, Dict[int, int]]]:
         raise NotImplementedError
 
-    def getUnhashedFunctions(self, function_ids: Optional[List[int]] = None) -> List["FunctionEntry"]:
+    def getUnhashedFunctions(self, function_ids: Optional[List[int]] = None, only_function_ids=False) -> List["FunctionEntry"]:
         """Given a list of function_ids, return all FunctionEntry objects corresponding to these IDs if they do not have a minhash yet.
         Otherwise, return all FunctionEntry objects that do not have a minhash.
 
         Args:
             function_ids: (optional) a list of function_ids
+            only_function_ids: (optional) instead if FunctionEntry objects, only return function_ids
 
         Returns:
             a list of FunctionEntry objects without minhash
@@ -521,11 +564,12 @@ class StorageInterface:
         raise NotImplementedError
 
 
-    def getUniqueBlocks(self, sample_ids: Optional[List[int]] = None) -> Dict:
+    def getUniqueBlocks(self, sample_ids: Optional[List[int]] = None, progress_reporter=None) -> Dict:
         """Given a list of sample_ids, return all basic blocks that are only found in any of these samples (and no other samples in the storage)
 
         Args:
             sample_ids: (optional) a list of sample_ids
+            progress_reporter: (optional) might be passed by worker to inquiry progress of this DB only operation
 
         Returns:
             a dictionary with the isolated blocks
@@ -576,9 +620,9 @@ class StorageInterface:
             a dict containing signature indices used for bandhashing by band id
         """
         band_projection = {}
-        random.seed(self._config.STORAGE_BAND_SEED)
+        random.seed(self._storage_config.STORAGE_BAND_SEED)
         band_index = 0
-        for band_size, num_bands in self._config.STORAGE_BANDS.items():
+        for band_size, num_bands in self._storage_config.STORAGE_BANDS.items():
             for _ in range(num_bands):
                 index_sequence = [index for index in range(len(minhash.getMinHashInt()))]
                 random.shuffle(index_sequence)
